/*
The following licence has been copied from
http://www.opensource.org/licenses/mit-license.html

Copyright (c) 2000, Peter Arthur Loeb

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.palserv.XMidi;

import java.io.File;
import org.w3c.dom.Document;
import javax.xml.parsers.*;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * This is the helper class for the Sun JAXP parser.
 * This class allows the JAXP parser to be used with
 * the rest of the com.palserv.XMidi package.
*/
public class JAXP implements com.palserv.XMidi.Parser {

/**
 * The default public constructor must be public
 * so it can be invoked &quot;by name&quot;.
 * See the
 * {@link com.palserv.XMidi.Util#getDoc(String,boolean,String) Util.getDoc}
 * method for how it is used.
*/
	public JAXP() {}

/**
 * Parse the XML and return a DOM Document
 * @param String the filename of the XML document
*/
	public Document parseXML(String fid) {
		Document doc = null;

		try {
			DocumentBuilderFactory docBuilderFactory =
				DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder =
				docBuilderFactory.newDocumentBuilder();
			doc = docBuilder.parse (new File (fid));
		} catch (SAXParseException err) {
			System.out.println ("** Parsing error"
			+ ", line " + err.getLineNumber ()
			+ ", uri " + err.getSystemId ());
			System.out.println("   " + err.getMessage ());
			// print stack trace as below
		} catch (SAXException e) {
			Exception	x = e.getException ();
			((x == null) ? e : x).printStackTrace ();
		} catch (Throwable t) {
			t.printStackTrace ();
		}
		return doc;
	}

/**
 * Simply return a new (empty) Document
 * @return Document
*/
	public Document getNewDoc() {
		Document doc = null;
		DocumentBuilderFactory docBuilderFactory = null;
		DocumentBuilder docBuilder = null;

		try {
			docBuilderFactory = DocumentBuilderFactory.newInstance();
		} catch (FactoryConfigurationError err) {
			System.out.println ("** Factory Configuration error"
			+ err.getMessage());
		}

		try {
			docBuilder = docBuilderFactory.newDocumentBuilder();
		} catch (ParserConfigurationException err) {
			System.out.println ("** new doc builder error"
			+ err.getMessage());
		}

		doc = docBuilder.newDocument();
		return doc;
	}
}
