/*
The following licence has been copied from
http://www.opensource.org/licenses/mit-license.html

Copyright (c) 2000, Peter Arthur Loeb

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.palserv.XMidi;

import fr.loria.xml.XDOM;
import fr.loria.xml.XDocument;
import org.xml.sax.InputSource;
import org.w3c.dom.Document;
import java.io.*;

/**
 * This class allows the silfide XML parser
 * to be used with the XMidi package.
 * <p>
 * I suggest that this class be used as a model for making
 * classes to allow inclusion of external parsers.
 * <p>
 * This class must implement com.palserv.XMidi.Parser
 * @author Peter Arthur Loeb
*/
public class XSilfide implements Parser {

/**
 * The default constructor (which takes no arguments
 * and does nothing) is necessary.
 * It is used by the
 * {@link com.palserv.XMidi.Util#getDoc(String,boolean,String) Util.getDoc}
 * method to load this class.
*/
	public XSilfide() {}

/**
 * This is a conversion routine.  It is invoked from
 * the
 * {@link com.palserv.XMidi.Util#getDoc(String,boolean,String) Util.getDoc}
 * method to parse the XML file for the XM class.
 * Although the argument is meant to be a file id, it
 * doesn't need to be, it can be any string needed.
 * It will come from the first non-option argument
 * to the XM class.
 * @param String fid - file id of XML file to parse.
 * @return Document as in com.w3c.dom.Document or a sub-class.
*/
	public Document parseXML(String fid) {
		XDOM xd = new XDOM();
		Document xDoc = null;

		FileInputStream fis = null;
		try {
			fis = new FileInputStream(fid);
		} catch(FileNotFoundException e) {
			System.err.println("Error opening: "+fid+": "+e.getMessage());
		}

		InputSource xis = new InputSource(fis);
		xDoc = (Document)xd.parseDocument(xis);

		try {
			fis.close();
		} catch(IOException e) {
			System.err.println("Error closing: "+fid+": "+e.getMessage());
		}

		return xDoc;
	}

/**
 * Create a new (empty) Document.
 * It is invoked from the
 * {@link com.palserv.XMidi.Util#getDoc(String,boolean,String) Util.getDoc}
 * method to obtain a new Document for the MX class.
 * @return Document
*/
	public Document getNewDoc() {
		XDOM xd = new XDOM();
		Document xDoc = (Document)new XDocument(xd);
		return xDoc;
	}
}
