/*
The following licence has been copied from
http://www.opensource.org/licenses/mit-license.html

Copyright (c) 2000, Peter Arthur Loeb

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.palserv.XMidi;

import org.w3c.dom.Document;
import java.lang.reflect.*;
import org.xml.sax.Attributes;

/**
internal utilities
<p>
NOT FOR EXTERNAL USE
<p>
I have made it public because it contains
some methods which should be included in
the javadoc.
*/
public class Util {
	static boolean gotParser = false;
	static Parser parser = null;

	static String getAttr(Attributes attrs, String qName) {
		String ret = null;
		for (int i = 0; i < attrs.getLength(); i ++) {
			if (attrs.getQName(i).equals(qName)) {
				ret = attrs.getValue(i);
				break;
			}
		}
		return ret;
	}

/**
 * Get an instance of an object from its name.
 * @return the object instance.
 */
	public static Object getInstanceFromName(String name) {
		Class cls = null;
		Constructor cnst = null;
		Object obj = null;

		if (name == null) {
			Util.rpt("Util.getInstanceFromName: name is null");
		}
		try {
			cls = Class.forName(name);
		} catch(ClassNotFoundException e) {
			Util.ept(e, "Util.getInstanceFromName: Class not found: "+name);
		}
		try {
			cnst = cls.getConstructor(new Class[]{});
		} catch (NoSuchMethodException e) {
			Util.ept(e,"Util.getInstanceFromName: Error no constructor for: "+name);
		} catch (SecurityException e) {
			Util.ept(e,"Util.getInstanceFromName: security execption: "+name);
		}
		try {
			obj = cnst.newInstance(new Object[]{});
		} catch (Exception e) {
			Util.ept(e,"Util.getInstanceFromName: Error creating: "+name);
		}
		return obj;
	}

/**
convert hex string to byte array
*/
	static byte[] hexToBa(String h, int len) {
		int l = h.length();
		int cnt = 0;
		int bs = 0;
		int bc = 0;
		byte[] ba = new byte[len];
		for (int i = 0; i < l; i ++) {
			String s = h.substring(i,i+1);
			int x = " \n\t\r".indexOf(s);
			if (x != -1) {
				continue;
			}
			int tmp = "0123456789ABCDEF".indexOf(s.toUpperCase());
			if (bc == 0) {
				bs = tmp;
			} else if (bc == 1) {
				bs <<= 4;
				bs |= tmp;
			} else {
				rpt("hexToBa: internal error");
			}
			bc ++;
			if (bc >= 2) {
				ba[cnt] = (byte)bs;
				cnt ++;
				bc = 0;
			}
		}
		if (bc != 0) {
			rpt("un-even number of hex digits");
		}
		if (cnt != len) {
			rpt("hexToBa: count ("+cnt+") != length ("+len+")");
		}
		return ba;
	}

/**
convert byte array to hex string
*/
	static String baToHex(byte[] b, int strt, int end) {
		int l = end - strt + 1;
		if (b.length < end) {
			try {
				rpts("Util.baToHex: length error; b.length="+b.length+", strt="+
					strt+", end="+end+", l="+l);
//					  strt+", end="+end+", l="+l+", b="+new String(b));
			} catch(Exception e) {
				e.printStackTrace();
			}
		}
		StringBuffer sb = new StringBuffer("");
		for (int i = 0; i < l; i ++) {
			int t = getUnsignedByte(b[strt + i]);
			int a = t / 16;
			int aa = t % 16;
			sb.append("0123456789ABCDEF".substring(a,a + 1));
			sb.append("0123456789ABCDEF".substring(aa,aa + 1));
		}
		return sb.toString();
	}

/**
convert int to byte array of length c
*/
	static byte[] intToBa(int n, int c) {
		byte[] b = new byte[c];
		int t = n;
		for (int i = 0; i < c; i ++) {
			int j = c - i - 1;
			int k = t % 256;
			b[j] = (byte)k;
			t /= 256;
		}
		if (t > 0) {
			rpt("intToBa: t is "+t+", n = "+n+", c = "+c);
		}
		return b;
	}

/**
Exception reporting
*/
	static void ept(Exception e, String s) {
		System.err.println(s);
		System.err.println("Exception message: "+e.getMessage());
		System.exit(0);
	}

/**
Error (non-exception) reporting
*/
	static void rpt(String s) {
		System.err.println("Error: "+s);
		System.exit(0);
	}

	static void rpts(String s) throws Exception {
		System.err.println("Error: "+s);
		throw new Exception(s);
	}

/**
convert byte array to int
*/
	static int baToInt(byte[] b, int strt, int end) {
		if (end > b.length - 1) {
			rpt("baToInt: strt = "+strt+", end = "+end+
				", b.length = "+b.length);
		}
		int l = end - strt + 1;
		int i = 0;
		for (int j = 0; j < l; j ++) {
			int p = strt + l - j - 1;
			// get int value of unsigned byte into k
			if (p > b.length - 1) {
				rpt("baToInt: p = "+p+", strt = "+strt+
					", end = "+end+", l = "+l+", j = "+j+
					", i = "+i);
			}
			int k = getUnsignedByte(b[p]);
			int n = pow(256,j);
			i += n * k;
		}
		return i;
	}

/**
convert byte (unsigned) to int
*/
	static int getUnsignedByte(byte b) {
		int t = 0;
		if ((b & 128) == 128) {
			t = 1;
		}
		b &= 127;
		int k = b;
		k += t * 128;
		return k;
	}

/**
compute b to the e power (b ** e)
*/
	static int pow(int b, int e) {
		int a = 1;
		for (int i = 0; i < e; i ++) {
			a *= b;
		}
		return a;
	}

/**
initialization for -v (verbose) stuff
*/
	static void vInit() {
	}

/**
 * This routine parses &quot;command line&quot;
 * arguments and returns a structure (class)
 * containing the needed data.
 * <p>
 * There are two kinds of arguments, options
 * and non-options.  Options begin with a hyphen
 * and non-options don't.  Options and non-options
 * may be mixed together and options may be in any
 * order.  The first non-option is the input fileid.
 * It is required, and should have a file extension
 * of xmi (for XM) or mid (for MX).
 * The second (and hopefully last)
 * non-option is the (optional) output fileid.
 * If it is not coded, we will use the input fileid,
 * changing the file extension to mid (for XM) or
 * xmi (for MX).
 * <p>
 * options:
 * <ul>
 * <li>-p<i>parser_id</i>
 * <p>
 * This is the name of the external parser class.
 * If the -q option is coded, then the
 * <i>parser_id</i>
 * is the (complete) <i>package-name.class-name</i>
 * for the class.
 * If the -q option is not coded, then the
 * <i>parser_id</i> is the name of a class in
 * this (com.palserv.XMidi) package, and should
 * be coded without the package name.
 * <li>-q
 * <p>If coded, this option means that the
 * <i>parser_id</i> coded in the &quot;-p&quot;
 * option is fully qualified.  If not coded,
 * this option means that the <i>parser_id</i>
 * coded in the &quot;-p&quot; is part of this
 * package.  The correct package identifier
 * will be prepended to the string.
 * <li>-d<i>debug_file</i>
 * <p>The fully qualified name of a file which
 * will be created to contain debugging data
 * for the most recent run.  As the name implies
 * this flag should only be used when the package
 * is being tested.  The output can be quite
 * large, and I am sure that there is a performance
 * penalty for using this feature.
 * <li>-v
 * <p>If coded, then include &quot;verbose&quot;
 * attributes.
 * This includes NAME and REGISTER on the NOTE_OFF,
 * NOTE_ON, and AFTER tags, and the NAME on the CONTROL
 * tag.
 * </ul>
 * @param String[] args - command line arguments (as in main)
*/
	public static CmdLineArgs getCmdLine(String[] args) {
		boolean qualified = false;
		CmdLineArgs cla = new CmdLineArgs();
		int noCnt = 0;
		/* validate command arguments */
		for (int i = 0; i < args.length; i ++) {
//spt("looking at argument: "+args[i]);
			if (args[i].substring(0,1).equals("-")) {
//spt("we have an option: "+args[i]);
				// do options
				if (args[i].substring(1,2).equals("p")) {
					cla.parserName = args[i].substring(2);
//spt("Setting parserName: "+cla.parserName);
				}
				if (args[i].substring(1,2).equals("d")) {
					cla.debugFile = args[i].substring(2);
//spt("Setting debugFile: "+cla.debugFile);
				}
				if (args[i].substring(1,2).equals("q")) {
					qualified = true;
//spt("Setting qualified: true");
				}
				if (args[i].substring(1,2).equals("v")) {
					cla.verbose = true;
//spt("Setting verbose: true");
				}

			} else {
				// do non option
				if (noCnt == 0) {
					cla.inFile = args[i];
				} else if (noCnt == 1) {
					cla.outFile = args[i];
				} else {
					rpt("too many non-option arguments");
				}
				noCnt ++;
			}
		}	// end for (each arg)

		if (!qualified) {
			cla.parserName = "com.palserv.XMidi."+cla.parserName;
		}

		if (noCnt == 0) {
			rpt("no input file specified");
		}
		if (noCnt == 1) {
			// no output file
			// find last period
			int p = cla.inFile.lastIndexOf(".");
			if (p == -1) {
				cla.outFile = cla.inFile+".mid";
			} else {
				String ext = cla.inFile.substring(p + 1);
				if (ext.equals("mid")) {
					rpt("invalid input extension: mid");
				}
				String fnam = cla.inFile.substring(0,p);
				cla.outFile = fnam+".mid";
			}
		}
		//spt("Input file: "+inFile);
		//spt("Output file: "+outFile);
		return cla;
	}

/**
 * This routine is used to invoke the external parser
 * helper class by name.
 * @param String fid - name of the file to parse
 * @param boolean doParse - if true, invoke parseXML to parse
 * the file; if false, invoke getNewDoc to obtain new (empty) Document
 * @return Document
*/
	static public Document getDoc(String fid, boolean doParse,
			String parserName) {
		if (!gotParser) {
			Class parserClass = null;
			Constructor parserConstructor = null;
			try {
	//spt("before Class.forName: "+parserName);
				parserClass = Class.forName(parserName);
			} catch (ClassNotFoundException e) {
				ept(e,"Error in Class.forName: "+parserName);
			}

	//spt("before getConstructor");
			try {
				parserConstructor = parserClass.getConstructor(
					new Class[]{});
			} catch (NoSuchMethodException e) {
				ept(e,"Error no such method: "+parserName);
			} catch (SecurityException e) {
				ept(e,"Error security: "+parserName);
			}
	//spt("before newInstance");
			try {
				parser = (Parser) parserConstructor.newInstance(
					new Object[]{});
			} catch (Exception e) {
				ept(e,"Error loading parser: "+parserName);
			}
			gotParser = true;
		}	// end if gotParser

		Document xDoc = null;
		if (doParse) {
			xDoc = parser.parseXML(fid);
		} else {
			xDoc = parser.getNewDoc();
		}

		if (xDoc == null) {
			rpt("Error in parser: " + parserName);
		}
//spt("back from parser");
		return xDoc;
	}

	static void spt(String s) {
		// do nuttin' till you hear from me
	}
}
