package com.palserv.XMidi;

import org.xml.sax.Attributes;

/**
 * Base class for all Tag classes.
 * This class should never be instantiated.
 * There are two methods for MX (midi to XMidi)
 * processing (createStartXML and createEndXML)
 * and three methods for XM (XMidi to midi) processing
 * (startElement, endElement and doContent).
 */
public class Tag implements iTag {
	String name = "Tag";

/**
 * Factory method to get a Tag by name.
 * @param tName - the tag name.
 * @return the correct Tag object.
 */
	public static Tag getTag(String tName) {
		String tagName = "com.palserv.XMidi.Tag"+tName;
		Tag tag = (Tag)Util.getInstanceFromName(tagName);
		return tag;
	}

/**
 * Handle the beginning of each element.
 * Invoked by the
 * {@link com.palserv.XMidi.XM#startElement(String,String,String,Attributes) XM.startElement}
 * method of XM, which implements the SAX interface.
 * @param attr - a list of attributes (org.xml.sax.Attributes)
 * @param state - the State object.
 */
	public void startElement(Attributes attr, State state) {
		Util.rpt("Tag.startElement: we should never be called! "+this.getClass().getName());
	}

/**
 * Handle the end of each element.
 * Invoked by the
 * {@link com.palserv.XMidi.XM#endElement(String,String,String) XM.endElement}
 * method of XM, which implements the SAX interface.
 * @param state - the State object.
 */
	public void endElement(State state) {}

/**
 * Create the XML start tag.
 * This is invoked from various places during MX
 * processing.
 * @param state - the State object.
 * @return the start tag.
 */
	public String createStartXML(State state) {
		return null;
	}

/**
 * Create the XML end tag, if needed.
 * This is invoked from various places during MX
 * processing.
 * @param state - the State object.
 * @return the end tag.
 */
	public String createEndXML(State state) {
		return Util.makeIndent(state)+"</"+getName()+">\n";
	}

/**
 * Get the tag name.
 * @return the tag name.
 */
	public String getName() {
		return name;
	}

/**
 * Handle the content for Tag sublcasses which have content.
 * (See the DTD).
 * This method is invoked from the
 * {@link com.palserv.XMidi.XM#characters(char[],int,int) XM.character}
 * method of the XM class, which implements the SAX interface for
 * content.
 * @param s - the content.
 * @param state - the State object.
 */
	public void doContent(String s, State state) {}
}

