package com.palserv.XMidi;

import org.xml.sax.Attributes;
import java.util.*;

/**
 * Implement the DELTA tag.
 * This class must be public in order for the
 * {@link com.palserv.XMidi.Tag#getTag(String) Tag.getTag}
 * method to work.
 * This is one of many Tag subclasses which are all similar.
 * Please see the
 * {@link com.palserv.XMidi.Tag Tag}
 * document and the
 * {@link com.palserv.XMidi.iTag iTag}
 * document for more details.
 */
public class TagDELTA extends Tag {
	private String name = "DELTA";

	public void startElement(Attributes attr, State state) {
		if (!state.chunkFlag) {
			Util.rpt("DELTA element only valid within a chunk");
		}
		if (!state.chunkType.equals("MTrk")) {
			Util.rpt("DELTA element must be in MTrk chunk");
		}
		if (!state.getParent().equals("MTrk")) {
			Util.rpt("DELTA element must be child of MTrk");
		}
		String dtime = Util.getAttr(attr,"DTIME");
		byte[] hdt = Util.hexToBa(dtime,4);
		byte[] dt = Util.intToDelta(hdt);
		Util.wpt(dt, state);
	}

	public String createStartXML(State state) {
		ArrayList<TagAttr> al = new ArrayList<TagAttr>();
		TagAttr dt = new TagAttr("DTIME", state.deltaLen);
		al.add(dt);
		String x = Util.makeTag(name, al, true, state);
		return x;
	}

	public String getName() {
		return name;
	}
}

