package com.palserv.XMidi;

import org.xml.sax.Attributes;

/**
 * Implement the EDATA tag.
 * This class must be public in order for the
 * {@link com.palserv.XMidi.Tag#getTag(String) Tag.getTag}
 * method to work.
 * This is one of many Tag subclasses which are all similar.
 * Please see the
 * {@link com.palserv.XMidi.Tag Tag}
 * document and the
 * {@link com.palserv.XMidi.iTag iTag}
 * document for more details.
 */
public class TagEDATA extends Tag {
	private String name = "EDATA";

	public void startElement(Attributes attr, State state) {
		if (!state.chunkFlag) {
			Util.rpt("EDATA element only valid within a chunk");
		}
		if (!state.chunkType.equals("MTrk")) {
			Util.rpt("EDATA element must be in MTrk chunk");
		}
		if (!state.getParent().equals("DELTA") &&
				!state.getParent().equals("STATUS")) {
			Util.rpt("STATUS element must be child of " +
				"DELTA or STATUS");
		}
		state.content = null;
	}

	public String createStartXML(State state) {
		StringBuffer bth = new StringBuffer("");
		bth.append(Util.makeTag(name, null, true, state));
		state.indent ++;
		bth.append(Util.makeIndent(state)+state.edata+"\n");
		state.indent --;
		return bth.toString();
	}

	public String getName() {
		return name;
	}

	public void doContent(String xdata, State state) {
		if (!state.chunkFlag) {
			Util.rpt("EDATA element only valid within CHUNK");
		}
		if (state.edFlag) {
			state.edFlag = false;
		} else {
			if (xdata == null || xdata.equals("")) {
				return;
			}
			if (xdata.equals("No data") || xdata.equals("null")) {
				return;
			}
			if (state.content == null) {
				state.content = xdata;
			} else {
				state.content += xdata;
			}
		}
	}

	public void endElement(State state) {
		String xdata = state.content;
		if (xdata == null || xdata.equals("")) {
			return;
		}
		int l = xdata.length();
		if (l % 2 != 0 || l / 2 != state.slen) {
			Util.rpt("EDATA content invalid: l = "+l+"; slen = "+state.slen+
				"; xdata = "+xdata);
		}
		Util.writeHex(xdata,state.slen, state);
		state.content = null;
	}
}
