package com.palserv.XMidi;

import org.xml.sax.Attributes;

/**
 * Implement the HEXDATA tag.
 * This class must be public in order for the
 * {@link com.palserv.XMidi.Tag#getTag(String) Tag.getTag}
 * method to work.
 * This is one of many Tag subclasses which are all similar.
 * Please see the
 * {@link com.palserv.XMidi.Tag Tag}
 * document and the
 * {@link com.palserv.XMidi.iTag iTag}
 * document for more details.
 */
public class TagHEXDATA extends Tag {
	private String name = "HEXDATA";

	public String createStartXML(State state) {
		StringBuffer bth = new StringBuffer("");
		bth.append(Util.makeTag(name, null, true, state));
		int bpl = 32;
		int r = state.chunkLen % bpl;
		int n = state.chunkLen / bpl;
		state.indent ++;
		for (int i = 0; i < n; i ++) {
			int strt = i * bpl;
			bth.append(Util.makeIndent(state)+
				Util.baToHex(state.dta,strt,strt + bpl - 1)+"\n");
		}
		if (r > 0) {
			int strt = n * bpl;
			bth.append(Util.makeIndent(state)+
				Util.baToHex(state.dta,strt,strt + r - 1)+"\n");
		}
		state.indent --;
		return bth.toString();
	}

	public String getName() {
		return name;
	}


	public void startElement(Attributes attr, State state) {
		if (!state.getParent().equals("CHUNK")) {
			Util.rpt("HEXDATA element only valid within CHUNK");
		}
		state.content = null;
	}

	public void doContent(String xdata, State state) {
		if (xdata == null) {
			Util.rpt("null value for CHUNK->HEXDATA");
		}
		if (state.content == null) {
			state.content = xdata;
		} else {
			state.content += xdata;
		}
/*
		int l = xdata.length();
		if (l % 2 != 0) {
			Util.rpt("HEXDATA trying to write un-even number of bytes: "+l+": "+xdata);
		}
		Util.writeHex(xdata,l / 2, state);
*/
	}

	public void endElement(State state) {
		String xdata = state.content;
		if (xdata == null || xdata.equals("")) {
			return;
		}
		int l = xdata.length();
		if (l % 2 != 0) {
			Util.rpt("HEXDATA content invalid: l = "+l+"; xdata = "+xdata);
		}
		Util.writeHex(xdata,l / 2, state);
		state.content = null;
	}
}
