package com.palserv.XMidi;

import org.xml.sax.Attributes;

/**
 * Implement the PPNQ tag.
 * This class must be public in order for the
 * {@link com.palserv.XMidi.Tag#getTag(String) Tag.getTag}
 * method to work.
 * This is one of many Tag subclasses which are all similar.
 * Please see the
 * {@link com.palserv.XMidi.Tag Tag}
 * document and the
 * {@link com.palserv.XMidi.iTag iTag}
 * document for more details.
 */
public class TagPPNQ extends Tag {
	private String name = "PPNQ";

	public String createStartXML(State state) {
		StringBuffer x = new StringBuffer(Util.makeTag(name, null, true, state));
		state.indent ++;
		String pnq = Util.baToHex(state.dta,4,5);
		x.append(Util.makeIndent(state)+pnq+"\n");
		state.indent --;
		return x.toString();
	}

	public String getName() {
		return name;
	}

	public void startElement(Attributes attr, State state) {}

	public void doContent(String pnq, State state) {
		if (!state.chunkFlag) {
			Util.rpt("PPNQ element only valid within a chunk");
		}
		if (!state.chunkType.equals("MThd")) {
			Util.rpt("PPNQ element must be in MThd chunk");
		}
		if (pnq.length() != 4) {
			Util.rpt("PPNQ content must be four hex (0-9,A-F) characters; is "+pnq);
		}
		Util.writeHex(pnq,2, state);
	}
}
