package com.palserv.XMidi;

import org.xml.sax.Attributes;
import java.util.*;

/**
 * Implement the STATUS tag.
 * This class must be public in order for the
 * {@link com.palserv.XMidi.Tag#getTag(String) Tag.getTag}
 * method to work.
 * This is one of many Tag subclasses which are all similar.
 * Please see the
 * {@link com.palserv.XMidi.Tag Tag}
 * document and the
 * {@link com.palserv.XMidi.iTag iTag}
 * document for more details.
 */
public class TagSTATUS extends Tag {
	private String name = "STATUS";

	public void startElement(Attributes attr, State state) {
		if (!state.chunkFlag) {
			Util.rpt("STATUS element only valid within a chunk");
		}
		if (!state.chunkType.equals("MTrk")) {
			Util.rpt("STATUS element must be in MTrk chunk");
		}
		if (!state.getParent().equals("DELTA")) {
			Util.rpt("STATUS element must be child of DELTA");
		}
		String sval = Util.getAttr(attr, "SVAL");
		Util.writeHex(sval,1, state);
		String sl = Util.getAttr(attr, "SLEN");
		state.slen = Util.stringToInt(sl);
		if (sval.equals("FF")) {
			String nmd = Util.getAttr(attr, "SNMT");
			Util.writeHex(nmd,1, state);
			byte[] hdt = Util.intToBa(state.slen,4);
			byte[] xdt = Util.intToDelta(hdt);
			Util.wpt(xdt, state);
			if (state.slen == 0) {
				state.edFlag = true;
			}
		} else if (sval.equals("F0")) {
			byte[] hdt = Util.intToBa(Util.stringToInt(sl),4);
			Util.wpt(Util.intToDelta(hdt), state);
		} else if (sval.equals("F6") | sval.equals("F8") |
				sval.equals("FA") | sval.equals("FB") |
				sval.equals("FC") | sval.equals("FE")) {
			state.edFlag = true;
		}
	}

	public String createStartXML(State state) {
		ArrayList<TagAttr> attrs = new ArrayList<TagAttr>();
		TagAttr snam = new TagAttr("SNAM", state.snam);
		attrs.add(snam);
		TagAttr sval = new TagAttr("SVAL", state.sval);
		attrs.add(sval);
		TagAttr slen = new TagAttr("SLEN", ""+state.slen);
		attrs.add(slen);
		if (!state.nmData.equals("")) {
			TagAttr snmt = new TagAttr("SNMT", state.nmData);
			attrs.add(snmt);
		}
		String x = Util.makeTag(name, attrs, true, state);
		return x;
	}


	public String getName() {
		return name;
	}
}
