/*
The following licence has been copied from
http://www.opensource.org/licenses/mit-license.html

Copyright (c) 2000, Peter Arthur Loeb

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.palserv.XMidi;

import org.w3c.dom.Document;
import java.lang.reflect.*;
import org.xml.sax.Attributes;
import java.io.*;
import java.util.*;

/**
internal utilities
<p>
NOT FOR EXTERNAL USE
<p>
I have made it public because it contains
some methods which should be included in
the javadoc.
*/
public class Util {
//	  static boolean gotParser = false;
//	  static Parser parser = null;

	static int readBin(byte[] bytes, String id, FileInputStream fis) {
		try {
			int r = fis.read(bytes);
			if (r == -1) {
				return -1;
			}
			if (r < bytes.length) {
				Util.rpt("error getting "+id);
			}
		} catch(IOException e) {
			Util.ept(e,"IO exception getting "+id);
		}
		return 0;
	}

	static String makeTag(String tName, ArrayList<TagAttr> attrs,
			boolean hasContent, State state) {
		String t = makeIndent(state)+"<"+tName;
		if (attrs != null) {
			for (int i = 0; i < attrs.size(); i ++) {
				TagAttr ta = attrs.get(i);
				t += " "+ta.name+"=\""+ta.value+"\"";
			}
		}
		if (hasContent) {
			t += ">";
		} else {
			t += "/>";
		}
		return t+"\n";
	}

	static String makeIndent(State state) {
		String n = "";
		for (int i = 0; i < state.indent; i ++) {
			n += "    ";
		}
		return n;
	}

/**
convert byte array to delta time
a delta time is expressed as a
byte array, length unknown (4 or less)
*/
	static byte[] intToDelta(byte[] t) {
/*
from fullword binary value to variable (midi) value:
- split number into 5 bit groups of 4 7 7 7 7
- just deal with the four groups of 7 (ignore the 4)
- put each group of 7 into a byte
- case 1: whole full word is zero.
	return one byte of zero.
- case 2: some non-zero bit (after first four bits)
	- work from left to right
	- throw away bytes which are zero until non-zero byte
		encountered
	- turn on hi-ord bit on all but last byte
*/
		int i1 = Util.baToInt(t,0,t.length - 1);
//spt("intToDelta: i1 = "+i1+", t = "+Util.baToHex(t,0,t.length - 1));
		if (i1 == 0) {
			byte[] b = new byte[1];
			b[0] = 0;
			return b;
		}
		int i2 = i1;
		byte[] b1 = new byte[4];
		for (int i = 0; i < 4; i ++) {
			int j = 4 - i - 1;
			int k = i2 % 128;
			i2 /= 128;
			b1[j] = (byte)k;
		}
		int j = -1;
		boolean bFlag = false;
		for (int i = 0; i < 4; i ++) {
			if (bFlag) { continue; }
			if (b1[i] != 0) {
				j = i;
				bFlag = true;
			}
		}
		int k = 4 - j;
		byte[] b2 = new byte[k];
		for (int i = 0; i < k; i ++) {
			b2[i] = b1[i + j];
			if (i != (k - 1)) {
				b2[i] |= 128;
			}
		}
		return b2;
	}

	static String fixString(String s) {
		byte[] b = s.getBytes();
		for (int i = 0; i < b.length; i ++) {
			if (b[i] > 127) {
				try {
					rpts("Util.fixString: invalid ascii char at index "+i+": "+s);
				} catch(Throwable t) {
					t.printStackTrace();
				}
			}
		}
		int start = -1;
		for (int i = 0; i < b.length; i ++) {
			if (b[i] > 127) {
				try {
					rpts("Util.fixString: invalid ascii char at index "+i+": "+s);
				} catch(Throwable t) {
					t.printStackTrace();
				}
			}
			if (b[i] > 32) {
				start = i;
				break;
			}
		}
		if (start == -1) {
			return null;
		}
		int end = -1;
		for (int i = b.length - 1; i >= 0; i ++) {
			if (b[i] > 32) {
				end = i;
				break;
			}
		}
		if (end == -1) {
			Util.rpt("Util.fixString: invalid end data: "+s);
		}
		int l = end - start + 1;
		String t = new String(b, start, l);
		return t;
	}

	static int stringToInt(String s) {
		Integer i = null;
		try {
			i = new Integer(s);
		} catch(NumberFormatException e) {
			rpt("Util.stringToInt: bad data: "+s);
		}
		return i.intValue();
	}

	static String getAttr(Attributes attrs, String qName) {
		String ret = null;
		for (int i = 0; i < attrs.getLength(); i ++) {
			if (attrs.getQName(i).equals(qName)) {
				ret = attrs.getValue(i);
				break;
			}
		}
		return ret;
	}


	static void writeString(String s, int len, State state) {
		int l = s.length();
		if (l != len) {
			Util.rpt("writeString; string length ("+l+") != expected length ("+len+")");
		}
		wpt(s.getBytes(), state);
	}

	static void writeHex(String h, int len, State state) {
		int l = h.length();
		int cnt = 0;
		int bs = 0;
		int bc = 0;
		for (int i = 0; i < l; i ++) {
			String s = h.substring(i,i+1);
			int x = " \n\t\r".indexOf(s);
			if (x != -1) {
				continue;
			}
			int tmp = "0123456789ABCDEF".indexOf(s.toUpperCase());
			if (bc == 0) {
				bs = tmp;
			} else if (bc == 1) {
				bs <<= 4;
				bs |= tmp;
			} else {
				Util.rpt("writeHex; internal error");
			}
			bc ++;
			if (bc >= 2) {
				wpt(bs, state);
				cnt ++;
				bc = 0;
			}
		}
		if (bc != 0) {
			try {
				Util.rpts("un-even number of hex digits");
			} catch(Throwable t) {
				t.printStackTrace();
			}
		}
/*
spt("writeHex: writing outside loop: "+cnt);
		wpt(bs);
		cnt ++;
*/
		if (cnt != len) {
			try {
				Util.rpts("writeHex count ("+cnt+") != length ("+len+")");
			} catch(Throwable t) {
				t.printStackTrace();
				Util.rpt("Getting out");
			}
		}
	}

	static void wpt(byte[] b, State state) {
		try {
		   state.fos.write(b);
		} catch(IOException e) {
			Util.ept(e,"Write error");
		}
	}

/**
overloaded wpt
*/
	static void wpt(int b, State state) {
		try {
		   state.fos.write(b);
		} catch(IOException e) {
			Util.ept(e,"Write error");
		}
	}

	static void writeFullWord(int f, State state) {
		byte[] b = Util.intToBa(f,4);
		wpt(b, state);
	}

	static void writeHalfWord(int h, State state) {
		byte[] b = Util.intToBa(h,2);
		wpt(b, state);
	}

/**
 * Get an instance of an object from its name.
 * @return the object instance.
 */
	public static Object getInstanceFromName(String name) {
		Class cls = null;
		Constructor cnst = null;
		Object obj = null;

		if (name == null) {
			Util.rpt("Util.getInstanceFromName: name is null");
		}
		try {
			cls = Class.forName(name);
		} catch(ClassNotFoundException e) {
			Util.ept(e, "Util.getInstanceFromName: Class not found: "+name);
		}
		try {
			cnst = cls.getConstructor(new Class[]{});
		} catch (NoSuchMethodException e) {
			Util.ept(e,"Util.getInstanceFromName: Error no constructor for: "+name);
		} catch (SecurityException e) {
			Util.ept(e,"Util.getInstanceFromName: security execption: "+name);
		}
		try {
			obj = cnst.newInstance(new Object[]{});
		} catch (Exception e) {
			Util.ept(e,"Util.getInstanceFromName: Error creating: "+name);
		}
		return obj;
	}

/**
convert hex string to byte array
*/
	static byte[] hexToBa(String h, int len) {
		int l = h.length();
		int cnt = 0;
		int bs = 0;
		int bc = 0;
		byte[] ba = new byte[len];
		for (int i = 0; i < l; i ++) {
			String s = h.substring(i,i+1);
			int x = " \n\t\r".indexOf(s);
			if (x != -1) {
				continue;
			}
			int tmp = "0123456789ABCDEF".indexOf(s.toUpperCase());
			if (bc == 0) {
				bs = tmp;
			} else if (bc == 1) {
				bs <<= 4;
				bs |= tmp;
			} else {
				rpt("hexToBa: internal error");
			}
			bc ++;
			if (bc >= 2) {
				ba[cnt] = (byte)bs;
				cnt ++;
				bc = 0;
			}
		}
		if (bc != 0) {
			rpt("un-even number of hex digits");
		}
		if (cnt != len) {
			rpt("hexToBa: count ("+cnt+") != length ("+len+")");
		}
		return ba;
	}

/**
convert byte array to hex string
*/
	static String baToHex(byte[] b, int strt, int end) {
		int l = end - strt + 1;
		if (b.length < end) {
			try {
				rpts("Util.baToHex: length error; b.length="+b.length+", strt="+
					strt+", end="+end+", l="+l);
//					  strt+", end="+end+", l="+l+", b="+new String(b));
			} catch(Exception e) {
				e.printStackTrace();
			}
		}
		StringBuffer sb = new StringBuffer("");
		for (int i = 0; i < l; i ++) {
			int t = getUnsignedByte(b[strt + i]);
			int a = t / 16;
			int aa = t % 16;
			sb.append("0123456789ABCDEF".substring(a,a + 1));
			sb.append("0123456789ABCDEF".substring(aa,aa + 1));
		}
		return sb.toString();
	}

/**
 * convert byte array to &quot;dump&quot; style string.
 * This means using periods (".") for unprintable characters.
 */
	static String baToString(byte[] b, int strt, int end) {
		int l = end - strt + 1;
		if (b.length < end) {
			try {
				rpts("Util.baToString: length error; b.length="+b.length+", strt="+
					strt+", end="+end+", l="+l);
			} catch(Exception e) {
				e.printStackTrace();
			}
		}
		StringBuffer sb = new StringBuffer("");
		for (int i = 0; i < l; i ++) {
			int t = getUnsignedByte(b[strt + i]);
			if (t < 20) {
				sb.append(".");
			} else {
				sb.append(new String(b, strt + i, 1));
			}
		}
		return sb.toString();
	}
/**
convert int to byte array of length c
*/
	static byte[] intToBa(int n, int c) {
		byte[] b = new byte[c];
		int t = n;
		for (int i = 0; i < c; i ++) {
			int j = c - i - 1;
			int k = t % 256;
			b[j] = (byte)k;
			t /= 256;
		}
		if (t > 0) {
			rpt("intToBa: t is "+t+", n = "+n+", c = "+c);
		}
		return b;
	}

/**
Exception reporting
*/
	static void ept(Exception e, String s) {
		System.err.println(s);
		System.err.println("Exception message: "+e.getMessage());
		System.exit(0);
	}

/**
Error (non-exception) reporting
*/
	static void rpt(String s) {
//		  State.getState().pw.close();
//		  System.err.println("Called from class: "+getClass().getName());
		System.err.println("Error: "+s);
		System.exit(0);
	}

	static void rpts(String s) throws Exception {
		System.err.println("Error: "+s);
		throw new Exception(s);
	}

/**
 * convert reverse byte array (intel) to int
 */
	static int reverseBaToInt(byte[] b, int strt, int end) {
		if (end > b.length - 1) {
			rpt("reverseBaToInt: strt = "+strt+", end = "+end+
				", b.length = "+b.length);
		}
		int l = end - strt + 1;
		byte[] rev = new byte[l];
		for (int i = 0; i < l; i ++) {
			int j = l - i -1;
			rev[j] = b[strt + i];
		}
		return baToInt(rev, 0, l - 1);
	}

/**
convert byte array to int
*/
	static int baToInt(byte[] b, int strt, int end) {
		if (end > b.length - 1) {
			rpt("baToInt: strt = "+strt+", end = "+end+
				", b.length = "+b.length);
		}
		int l = end - strt + 1;
		int i = 0;
		for (int j = 0; j < l; j ++) {
			int p = strt + l - j - 1;
			// get int value of unsigned byte into k
			if (p > b.length - 1) {
				rpt("baToInt: p = "+p+", strt = "+strt+
					", end = "+end+", l = "+l+", j = "+j+
					", i = "+i);
			}
			int k = getUnsignedByte(b[p]);
			int n = pow(256,j);
			i += n * k;
		}
		return i;
	}

/**
convert byte (unsigned) to int
*/
	static int getUnsignedByte(byte b) {
		int t = 0;
		if ((b & 128) == 128) {
			t = 1;
		}
		b &= 127;
		int k = b;
		k += t * 128;
		return k;
	}

/**
compute b to the e power (b ** e)
*/
	static int pow(int b, int e) {
		int a = 1;
		for (int i = 0; i < e; i ++) {
			a *= b;
		}
		return a;
	}

/**
initialization for -v (verbose) stuff
*/
	static void vInit() {
	}

/**
 * This routine parses &quot;command line&quot;
 * arguments and returns a structure (class)
 * containing the needed data.
 * <p>
 * There are two kinds of arguments, options
 * and non-options.  Options begin with a hyphen
 * and non-options don't.  Options and non-options
 * may be mixed together and options may be in any
 * order.  The first non-option is the input fileid.
 * It is required, and should have a file extension
 * of xmi (for XM) or mid (for MX).
 * The second (and hopefully last)
 * non-option is the (optional) output fileid.
 * If it is not coded, we will use the input fileid,
 * changing the file extension to mid (for XM) or
 * xmi (for MX).
 * <p>
 * options:
 * <ul>
 * <li>-v
 * <p>If coded, then include &quot;verbose&quot;
 * attributes.
 * This includes NAME and REGISTER on the NOTE_OFF,
 * NOTE_ON, and AFTER tags, and the NAME on the CONTROL
 * tag.
 * </ul>
 * @param args - command line arguments (as in main)
*/
	public static CmdLineArgs getCmdLine(String[] args) {
		CmdLineArgs cla = new CmdLineArgs();
		int noCnt = 0;
		/* validate command arguments */
		for (int i = 0; i < args.length; i ++) {
			if (args[i].substring(0,1).equals("-")) {
				// do options
				if (args[i].substring(1,2).equals("v")) {
					cla.verbose = true;
				} else if (args[i].substring(1,2).equals("t")) {
					cla.testMode = true;
				} else {
					rpt("Unknown option: "+args[i]);
				}
			} else {
				// do non option
				switch (noCnt) {
					case 0:
						cla.inFile = args[i];
						break;
					case 1:
						cla.outFile = args[i];
						break;
					default:
						rpt("too many non-option arguments: "+args[i]);
				}
				noCnt ++;
			}
		}	// end for (each arg)

		if (noCnt == 0) {
			rpt("no input or output file specified");
		}
		if (noCnt == 1) {
			rpt("no output file specified");
		}
		return cla;
	}

/**
 * This routine is used to invoke the external parser
 * helper class by name.
 * @param String fid - name of the file to parse
 * @param boolean doParse - if true, invoke parseXML to parse
 * the file; if false, invoke getNewDoc to obtain new (empty) Document
 * @return Document
*/
/*
	static public Document getDoc(String fid, boolean doParse,
			String parserName) {
		if (!gotParser) {
			Class parserClass = null;
			Constructor parserConstructor = null;
			try {
	//spt("before Class.forName: "+parserName);
				parserClass = Class.forName(parserName);
			} catch (ClassNotFoundException e) {
				ept(e,"Error in Class.forName: "+parserName);
			}

	//spt("before getConstructor");
			try {
				parserConstructor = parserClass.getConstructor(
					new Class[]{});
			} catch (NoSuchMethodException e) {
				ept(e,"Error no such method: "+parserName);
			} catch (SecurityException e) {
				ept(e,"Error security: "+parserName);
			}
	//spt("before newInstance");
			try {
				parser = (Parser) parserConstructor.newInstance(
					new Object[]{});
			} catch (Exception e) {
				ept(e,"Error loading parser: "+parserName);
			}
			gotParser = true;
		}	// end if gotParser

		Document xDoc = null;
		if (doParse) {
			xDoc = parser.parseXML(fid);
		} else {
			xDoc = parser.getNewDoc();
		}

		if (xDoc == null) {
			rpt("Error in parser: " + parserName);
		}
//spt("back from parser");
		return xDoc;
	}
*/
	static void spt(String s) {
		// do nuttin' till you hear from me
	}
}
