/*
The following licence has been copied from
http://www.opensource.org/licenses/mit-license.html

Copyright (c) 2005, Peter Arthur Loeb

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.palserv.XMidi;

import org.xml.sax.Attributes;
import java.util.*;

/**
 * Implement the CHUNK tag.
 * This class must be public in order for the
 * {@link com.palserv.XMidi.Tag#getTag(String) Tag.getTag}
 * method to work.
 * This is one of many Tag subclasses which are all similar.
 * Please see the
 * {@link com.palserv.XMidi.Tag Tag}
 * document and the
 * {@link com.palserv.XMidi.iTag iTag}
 * document for more details.
 */
public class TagCHUNK extends Tag {
	private String name = "CHUNK";

	public void startElement(Attributes attr, State state) {
		state.chunkFlag = true;
		state.chunkLen = Util.stringToInt(Util.getAttr(attr, "LENGTH"));
		String hh = Util.getAttr(attr, "HEXHDR");
		if (hh == null) {
			Util.rpt("TagCHUNK: HEXHDR is null");
		}
		if (hh.length() != 16) {
			Util.rpt("TagCHUNK: HEXHDR is not of length 16: is "+hh.length());
		}
		Util.writeHex(hh, 8, state);
/*
//		  String sd = new String(state.dta);
		if (state.dta == null) {
			Util.rpt("TagCHUNK: dta is null");
		}
		if (state.dta.length != state.chunkLen) {
			Util.rpt("TagCHUNK: dta is not of length "+state.chunkLen+": is "+state.dta.length);
		}
//		  Util.writeHex(new String(state.dta), state.chunkLen, state);
		Util.wpt(state.dta, state);
*/
	}

	public void endElement(State state) {
		state.chunkFlag = false;
	}

	public String createStartXML(State state) {
		ArrayList<TagAttr> attrs = new ArrayList<TagAttr>();
		TagAttr typ = new TagAttr("TYPE", name);
		attrs.add(typ);
		TagAttr len = new TagAttr("LENGTH", ""+state.chunkLen);
		attrs.add(len);
		TagAttr hdr = new TagAttr("HEXHDR", ""+Util.baToHex(state.hdr,0,7));
		attrs.add(hdr);
		if (state.verbose) {
			TagAttr shdr = new TagAttr("STRHDR", ""+Util.baToString(state.hdr,0,7));
			attrs.add(shdr);
		}
		StringBuffer x = new StringBuffer(Util.makeTag(name, attrs, true, state));
		state.indent ++;
		Tag hex = Tag.getTag("HEXDATA");
		x.append(hex.createStartXML(state));
		x.append(hex.createEndXML(state));
		if (state.verbose) {
			Tag strp = Tag.getTag("STRINGREP");
			x.append(strp.createStartXML(state));
			x.append(strp.createEndXML(state));
		}
		state.indent --;
		return x.toString();
	}

	public String getName() {
		return name;
	}
}

