/*
The following licence has been copied from
http://www.opensource.org/licenses/mit-license.html

Copyright (c) 2005, Peter Arthur Loeb

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.palserv.XMidi;

import org.xml.sax.Attributes;
import java.util.*;

/**
 * Implement the DELTA tag.
 * This class must be public in order for the
 * {@link com.palserv.XMidi.Tag#getTag(String) Tag.getTag}
 * method to work.
 * This is one of many Tag subclasses which are all similar.
 * Please see the
 * {@link com.palserv.XMidi.Tag Tag}
 * document and the
 * {@link com.palserv.XMidi.iTag iTag}
 * document for more details.
 */
public class TagDELTA extends Tag {
	private String name = "DELTA";

	public void startElement(Attributes attr, State state) {
		if (!state.chunkFlag) {
			Util.rpt("DELTA element only valid within a chunk");
		}
		if (!state.chunkType.equals("MTrk")) {
			Util.rpt("DELTA element must be in MTrk chunk");
		}
		if (!state.getParent().equals("MTrk")) {
			Util.rpt("DELTA element must be child of MTrk");
		}
		String dtime = Util.getAttr(attr,"DTIME");
		byte[] hdt = Util.hexToBa(dtime,4);
		byte[] dt = Util.intToDelta(hdt);
		Util.wpt(dt, state);
	}

	public String createStartXML(State state) {
		ArrayList<TagAttr> al = new ArrayList<TagAttr>();
		TagAttr dt = new TagAttr("DTIME", state.deltaLen);
		al.add(dt);
		if (state.verbose) {
			int len = state.deltaLen.length() / 2;
			int dlt = Util.baToInt(Util.hexToBa(state.deltaLen, len), 0, len - 1);
			TagAttr dec = new TagAttr("DECTIME", ""+dlt);
			al.add(dec);
		}
		String x = Util.makeTag(name, al, true, state);
		return x;
	}

	public String getName() {
		return name;
	}
}

