/*
The following licence has been copied from
http://www.opensource.org/licenses/mit-license.html

Copyright (c) 2005, Peter Arthur Loeb

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.palserv.XMidi;

import org.xml.sax.Attributes;

/**
 * Implement the EDATA tag.
 * This class must be public in order for the
 * {@link com.palserv.XMidi.Tag#getTag(String) Tag.getTag}
 * method to work.
 * This is one of many Tag subclasses which are all similar.
 * Please see the
 * {@link com.palserv.XMidi.Tag Tag}
 * document and the
 * {@link com.palserv.XMidi.iTag iTag}
 * document for more details.
 */
public class TagEDATA extends Tag {
	private String name = "EDATA";

	public void startElement(Attributes attr, State state) {
		if (!state.chunkFlag) {
			Util.rpt("EDATA element only valid within a chunk");
		}
		if (!state.chunkType.equals("MTrk")) {
			Util.rpt("EDATA element must be in MTrk chunk");
		}
		if (!state.getParent().equals("DELTA") &&
				!state.getParent().equals("STATUS")) {
			Util.rpt("STATUS element must be child of " +
				"DELTA or STATUS");
		}
		state.content = null;
	}

	public String createStartXML(State state) {
		StringBuffer bth = new StringBuffer("");
		bth.append(Util.makeTag(name, null, true, state));
		state.indent ++;
		bth.append(Util.makeIndent(state)+state.edata+"\n");
		state.indent --;
		return bth.toString();
	}

	public String getName() {
		return name;
	}

	public void doContent(String xdata, State state) {
		if (!state.chunkFlag) {
			Util.rpt("EDATA element only valid within CHUNK");
		}
/*
if (xdata != null) {
	Util.spt("TagEDATA.doContent: xdata = "+xdata);
} else {
	Util.spt("TagEDATA.doContent: xdata is null");
}
*/
		if (xdata == null || xdata.equals("")) {
			return;
		}
/*
		if (xdata.equals("No data") || xdata.equals("null")) {
			return;
		}
*/
		if (state.content == null) {
			state.content = xdata;
		} else {
			state.content += xdata;
		}
	}

	public void endElement(State state) {
		if (state.edFlag) {
			state.edFlag = false;
			return;
		}
		String xdata = state.content;
		if (xdata == null || xdata.equals("")) {
			return;
		}
		if (xdata.equals("No data")) {
			state.content = null;
			return;
		}
		int l = xdata.length();
		if (l % 2 != 0 || l / 2 != state.slen) {
			Util.rpt("EDATA content invalid: l = "+l+"; slen = "+state.slen+
				"; xdata = "+xdata+", snmt = "+state.nmData+", sval = "+state.sval);
		}
		Util.writeHex(xdata,state.slen, state);
		state.content = null;
	}
}
