/*
The following licence has been copied from
http://www.opensource.org/licenses/mit-license.html

Copyright (c) 2005, Peter Arthur Loeb

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.palserv.XMidi;

import org.xml.sax.Attributes;

/**
 * Implement the FORMAT tag.
 * This class must be public in order for the
 * {@link com.palserv.XMidi.Tag#getTag(String) Tag.getTag}
 * method to work.
 * This is one of many Tag subclasses which are all similar.
 * Please see the
 * {@link com.palserv.XMidi.Tag Tag}
 * document and the
 * {@link com.palserv.XMidi.iTag iTag}
 * document for more details.
 */
public class TagFORMAT extends Tag {
	String name = "FORMAT";

	public void startElement(Attributes attr, State state) {}

	public String createStartXML(State state) {
		StringBuffer x = new StringBuffer(Util.makeTag(name, null, true, state));
		state.indent ++;
		int midiFormat = Util.baToInt(state.dta,0,1);
		x.append(Util.makeIndent(state)+midiFormat+"\n");
		state.indent --;
		return x.toString();
	}

	public String getName() {
		return name;
	}

	public void doContent(String typ, State state) {
		if (!state.chunkFlag) {
			Util.rpt("FORMAT element only valid within a chunk");
		}
		if (!state.chunkType.equals("MThd")) {
			Util.rpt("FORMAT element must be in MThd chunk");
		}
		int midiFormat = Util.stringToInt(typ);
		Util.writeHalfWord(midiFormat, state);
	}
}
