/*
The following licence has been copied from
http://www.opensource.org/licenses/mit-license.html

Copyright (c) 2005, Peter Arthur Loeb

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.palserv.XMidi;

import org.xml.sax.Attributes;

/**
 * Implement the NOTE_ON tag.
 * This class must be public in order for the
 * {@link com.palserv.XMidi.Tag#getTag(String) Tag.getTag}
 * method to work.
 * This is one of many Tag subclasses which are all similar.
 * Please see the
 * {@link com.palserv.XMidi.Tag Tag}
 * document and the
 * {@link com.palserv.XMidi.iTag iTag}
 * document for more details.
 */
public class TagNOTE_ON extends TagNOTE_OFF {
	private String name = "NOTE_ON";

	public void startElement(Attributes attr, State state) {
		if (!state.chunkFlag) {
			Util.rpt("NOTE_ON element only valid within a chunk");
		}
		if (!state.chunkType.equals("MTrk")) {
			Util.rpt("NOTE_ON element must be in MTrk chunk");
		}
		if (!state.getParent().equals("CHANNEL")) {
			Util.rpt("NOTE_ON element must be child of CHANNEL");
		}
		String pitch = Util.getAttr(attr, "PITCH");
		Util.wpt(Util.stringToInt(pitch), state);
		String vel = Util.getAttr(attr, "VELOCITY");
		Util.wpt(Util.stringToInt(vel), state);
	}

	public String getName() {
		return name;
	}
}
