/*
The following licence has been copied from
http://www.opensource.org/licenses/mit-license.html

Copyright (c) 2005, Peter Arthur Loeb

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.palserv.XMidi;

import org.xml.sax.Attributes;
import java.util.*;

/**
 * Implement the XMidi tag.
 * This class must be public in order for the
 * {@link com.palserv.XMidi.Tag#getTag(String) Tag.getTag}
 * method to work.
 * This is one of many Tag subclasses which are all similar.
 * Please see the
 * {@link com.palserv.XMidi.Tag Tag}
 * document and the
 * {@link com.palserv.XMidi.iTag iTag}
 * document for more details.
 */
public class TagXMidi extends Tag {
	private String name = "XMidi";

	public void startElement(Attributes attr, State state) {
		// validate version
		String ver = Util.getAttr(attr, "VERSION");
		if (!ver.equals(Version._VERSION)) {
			Util.rpt("wrong version; expecting "+Version._VERSION+", got "+ver);
		}
	}

	public String createStartXML(State state) {
		ArrayList<TagAttr> attrs = new ArrayList<TagAttr>();
		TagAttr ta = new TagAttr("VERSION", Version._VERSION);
		attrs.add(ta);
		String x = Util.makeTag(name, attrs, true, state);
		return x;
	}

	public String getName() {
		return name;
	}
}
